<?php
/*
Plugin Name: Ping Watcher
Plugin URI: http://www.mittineague.com/dev/pw.php
Description: The Ping Watcher plugin can catch up to 100 ping errors. These are displayed in both a dashboard widget and on the plugin's Settings page. Great for tracking repeated ping failures so you can clean up your ping list.
Version: 1.0.1 RC
Author: Mittineague
Author URI: http://www.mittineague.com
*/

/*
* Notes
* Requires WordPress version 2.5+
* not compatible with Error Reporting plugin
*/

/*
* Change Log
*
* ver. 1.0.1 RC 01-Jun-2010
* - Dutch translation
*
* ver. 1.0.0 RC 27-Feb-2010
* - Belarusian translation
* - replaced deprecated user_role
* - minor gettext tweak
* 
* ver. Beta 0.2.1 21-Aug-2009
* - Swedish translation
* - non-default folder tweak
* - changed Version History to Changelog in readme
*
* ver. Beta 0.2.0 17-Jul-2009
* - Internationalization
* - German and Spanish translations
* - allow for non-default folder locations/names
* - capability check
*
* ver. Beta 0.1.0 01-Apr-2009
* - initial release
* - - derived from errorreporting.php 0.10.0
*/

/*
/--------------------------------------------------------------------\
|                                                                    |
| License: GPL                                                       |
|                                                                    |
| Ping Watcher Plugin - Logs ping errors.                            |
| Copyright (C) 2009, Mittineague, www.mittineague.com               |
| All rights reserved.                                               |
|                                                                    |
| This program is free software; you can redistribute it and/or      |
| modify it under the terms of the GNU General Public License        |
| as published by the Free Software Foundation; either version 2     |
| of the License, or (at your option) any later version.             |
|                                                                    |
| This program is distributed in the hope that it will be useful,    |
| but WITHOUT ANY WARRANTY; without even the implied warranty of     |
| MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the      |
| GNU General Public License for more details.                       |
|                                                                    |
| You should have received a copy of the GNU General Public License  |
| along with this program; if not, write to the                      |
| Free Software Foundation, Inc.                                     |
| 51 Franklin Street, Fifth Floor                                    |
| Boston, MA  02110-1301, USA                                        |   
|                                                                    |
\--------------------------------------------------------------------/
*/

/* nonce compatibility */
global $wp_version;
if ( version_compare($wp_version, "2.5", "<") )
{
	$mitt_pw_incompatible = 'Ping Watcher requires WordPress 2.5 or newer. 
		<a href="http://codex.wordpress.org/Upgrading_WordPress">Please update!</a>';
	exit ($mitt_pw_incompatible);
}

if ( ! defined( 'PLUGINDIR' ) )
	define( 'PLUGINDIR', 'wp-content/plugins' );

function mitt_pw_activate()
{
	$mitt_pw_ping_errs = array();
	add_option('mitt_pw_ping_errors', $mitt_pw_ping_errs);
	$mitt_pw_pe_len = 25;
	add_option('mitt_pw_ping_error_length', $mitt_pw_pe_len);
}

function mitt_pw_deactivate()
{
	remove_action('init', 'mitt_pw_pe_handler');
	remove_action('wp_dashboard_setup', 'add_pw_dashboard_widget');
	unregister_setting('mitt-pw-options', 'mitt_pw_pe_len', 'absint');
}

function mitt_pw_uninstall()
{
	delete_option('mitt_pw_ping_errors');
	delete_option('mitt_pw_ping_error_length');
}

function mitt_pw_add_settings_link($links, $file)
{
	if ( $file == plugin_basename(__FILE__) )
	{
		$pw_settings_link = "<a href='options-general.php?page=pingwatcher.php'>" . __('Settings', 'pingwatcher') . "</a>";
		array_unshift( $links, $pw_settings_link ); // before Deactivate | Edit
//		$links[] = $pw_settings_link; // after Deactivate | Edit
	}
	return $links;
}

function mitt_pw_admin_init()
{
	$pw_plugin_dir = dirname(plugin_basename(__FILE__));

	if (function_exists('load_plugin_textdomain'))
	{
		load_plugin_textdomain('pingwatcher', PLUGINDIR . '/' . $pw_plugin_dir . '/languages',
						 $pw_plugin_dir . '/languages' );

	}
	register_setting('mitt-pw-options', 'mitt_pw_pe_len', 'absint');
	add_filter('plugin_action_links', 'mitt_pw_add_settings_link', null, 2); 
}

function mitt_pw_dashboard_widget()
{
	$pw_ping_errs = get_option('mitt_pw_ping_errors');
	$num_p_errs = count($pw_ping_errs);

	if ($num_p_errs === 0)
	{
		_e('There are no recorded ping errors.', 'pingwatcher');
	}
	else
	{
		echo current($pw_ping_errs) . "<br />";
		for ($pe = 1; $pe < $num_p_errs; $pe++)
		{
			echo next($pw_ping_errs) . "<br />";
		}
	}
}

function add_pw_dashboard_widget()
{
	global $wp_meta_boxes;

	if ( function_exists('wp_add_dashboard_widget')
	 && !isset($wp_meta_boxes['dashboard']['normal']['core']['mitt_er_ping_error_widget']) )
	{
		wp_add_dashboard_widget('mitt_pw_ping_error_widget', 'Ping Watcher', 'mitt_pw_dashboard_widget');
	}
} 

function mitt_pw_css()
{
?>
<style type="text/css">
.postbox {
  margin-top: 1.5em;
 }
.postbox h3{
  margin: 0;
  padding: 0.5em;
  background-color: #eee;
 }
.postbox .inside {
  margin: 0;
  padding-left: 1em;
  background-color: #fff;
 }
.postbox .inside form p {
  margin-top: 0;
 }
.postbox .inside form .submit {
  padding: 0 0 0.5em 0;
 }
#mitt_pw_pe_len {
  margin-left: 1em;
  width: 2em;
  text-align: right;
 }
.submit {
  padding: 0.3em 0 0.3em 1.5em;
 }
</style>
<?php
}

function mitt_add_pw_page()
{
	global $pwnonce;
	if (function_exists('add_options_page'))
	{
		$mitt_pw_op = add_options_page('Ping Watcher', 'PingWatcher', 'manage_options', basename(__FILE__), 'mitt_pw_options_page');
		add_action("admin_head-$mitt_pw_op", 'mitt_pw_css');
	}
	$pwnonce = md5('pingwatcher');
}

function mitt_pw_options_page()
{
	if ( function_exists('current_user_can') && !current_user_can('manage_options') ) die;
?>
	<div class="wrap">
	<h2>Ping Watcher</h2>
<?php
	if ( ( is_plugin_active('errorreporting.php') ) || ( is_plugin_active('error-reporting/errorreporting.php') ) )
	{
		if( function_exists('mitt_err_options') ) set_error_handler('mitt_err_options');

		echo "<div class='fade'>
			<h3>" . sprintf(__('%s is installed and Activated.', 'pingwatcher'), '<a href=\'./options-general.php?page=errorreporting.php\'>Error Reporting</a>') . "</h3>
			<p>" . sprintf(__('%s is included as a part of that plugin.', 'pingwatcher'), 'Ping Watcher') . " " . __('Both can not be activated at the same time.', 'pingwatcher') . "<br />" . __('Please deactivate one or the other.', 'pingwatcher') . "</p>
		</div>";
	}
	else
	{
		global $pwnonce;
		$max_ping_saves = 100;	// arbitrary number
		register_setting('mitt-pw-options', 'mitt_pw_pe_len', 'absint');

		if ( isset($_POST['mitt_update_pw']) )
		{
			check_admin_referer('ping-watcher-update_' . $pwnonce, '_mitt_pw_u');

			if ( ( is_numeric($_POST['mitt_pw_pe_len']) ) && ($_POST['mitt_pw_pe_len'] >= 0) )
			{
				$mitt_pw_pe_len = absint($_POST['mitt_pw_pe_len']);
				$mitt_pw_pe_len = ($mitt_pw_pe_len > $max_ping_saves) ? $max_ping_saves : $mitt_pw_pe_len;
				update_option('mitt_pw_ping_error_length', $mitt_pw_pe_len);
			
				$mitt_pw_p_errs = get_option('mitt_pw_ping_errors');
				$num_p_errs = count($mitt_pw_p_errs);

				if ( $num_p_errs > $mitt_pw_pe_len )
				{
					$mitt_pw_p_errs = array_slice($mitt_pw_p_errs, 0, $mitt_pw_pe_len);
					update_option('mitt_pw_ping_errors', $mitt_pw_p_errs);
				}

		echo "<div id='message' class='updated fade'>
			<p>" . __('The number of Ping Errors to save has been updated.', 'pingwatcher') . "</p>
		</div>";

			}
		}

		if ( isset($_POST['mitt_pw_rem_pe']) )
		{
			check_admin_referer('ping-watcher-remove-errors_' . $pwnonce, '_mitt_pw_re');

			$mitt_pw_p_errs = array();
			update_option('mitt_pw_ping_errors', $mitt_pw_p_errs);

		echo "<div id='message' class='updated fade'>
			<p>" . __('All saved Ping Errors have been removed.', 'pingwatcher') . "</p>
		</div>";

		}

/* SHOW EXISTING PING ERRORS SECTION */
		echo "<div class='postbox'>
			<h3>" . __('Saved Ping Errors', 'pingwatcher') . "</h3>
			<div class='inside'>";

		$pw_ping_errs = get_option('mitt_pw_ping_errors');
		$num_p_errs = count($pw_ping_errs);

		function mitt_pw_page_prep($error_line)
		{
			$error_line = rtrim($error_line, '<br />');
			$error_line = str_replace('<br />', ' ~ ', $error_line);
			return $error_line;
		}

		if ($num_p_errs === 0)
		{
			echo "<p>" . __('There are no recorded ping errors.', 'pingwatcher') . "</p>";
		}
		else
		{	
			$pw_ping_errs = array_map('mitt_pw_page_prep', $pw_ping_errs);
			echo '<ul>';
			echo '<li>' . current($pw_ping_errs) . '</li>';

			for ($pe = 1; $pe < $num_p_errs; $pe++)
			{
				echo '<li>' . next($pw_ping_errs) . '</li>';
			}
			echo '</ul>';
		}
?>
			</div>
		</div>

<!-- /* UPDATE NUMBER OF PING ERRORS SECTION */ -->
		<div class="postbox">
<?php
			echo "<h3>" . __('Ping Error Setting', 'pingwatcher') . "</h3>";
?>
			<div class='inside'>
				<form method="post" action="<?php echo $_SERVER["REQUEST_URI"]; ?>">
<?php
		if ( function_exists('wp_nonce_field') )
			wp_nonce_field('ping-watcher-update_' . $pwnonce, '_mitt_pw_u');
		settings_fields('mitt-pw-options');

		echo "<label for='mitt_pw_pe_len'>" . __('Number of Ping Errors to Save:', 'pingwatcher') . " </label>";
?>
		<input id="mitt_pw_pe_len" name="mitt_pw_pe_len" type="text" value="<?php echo get_option('mitt_pw_ping_error_length'); ?>" />
<?php
		echo "<p>" . __('Setting a value lower than the existing number of saved ping errors will truncate the saved ping errors to that amount.', 'pingwatcher') . "<br />";
		printf(__('Maximum number is %d', 'pingwatcher'), $max_ping_saves);
?>
		</p>
		<p class="submit">
			<input type="submit" name="mitt_update_pw" value="<?php _e('Save Changes', 'pingwatcher') ?>" class="button" />
		</p>
				</form>
			</div>
		</div>

<!-- /* REMOVE ALL PING ERRORS SECTION */ -->
		<div class="postbox">
<?php
			echo "<h3>" . __('Remove all Saved Ping Errors', 'pingwatcher') . "</h3>";
?>
			<div class="inside">
				<form method="post" action="<?php echo $_SERVER["REQUEST_URI"]; ?>">
<?php
		if ( function_exists('wp_nonce_field') )
			wp_nonce_field('ping-watcher-remove-errors_' . $pwnonce, '_mitt_pw_re');

		echo "<p>" . __('This will remove all of the saved Ping Errors.', 'pingwatcher') . "</p>";
?>
		<p class="submit">
			<input type="submit" id="mitt_pw_rem_pe" name="mitt_pw_rem_pe" value="<?php _e('Remove Saved Ping Errors', 'pingwatcher') ?>" class="button" />
		</p>
				</form>
			</div>
		</div>
<?php
	}

/* PLUGIN NOTES */
		echo "<h2>" . __('Plugin Notes', 'pingwatcher') . "</h2>";
		echo "<ul>
			<li>" . sprintf(__('This plugin is not compatible with the %s plugin.', 'pingwatcher'), '<a href=\'http://www.mittineague.com/dev/er.php\'>Error Reporting</a>') . "<br />";
			printf(__('%s is included as a part of that plugin. Both can not be activated at the same time.', 'pingwatcher'), 'Ping Watcher');
			echo "</li>
			<li>" . __('Uninstalling this plugin using the WordPress plugin list page\'s "delete" will remove the plugin\'s options from the wp-options table, including any saved ping errors.', 'pingwatcher') . "</li>";
			echo "<li>" .  sprintf(__('If you do not want to see the widget on your dashboard, please go to dashboard\'s "Screen Options" and deselect %s', 'pingwatcher'), '&#34;Ping Watcher&#34') . "</li>";
			$owp_link = "<a href='./options-writing.php'>" . __('ping list', 'pingwatcher') . "</a>";
			echo "<li> *";
			printf(__('If a ping fails once, or even a few times, it does not necessarily mean that it should be removed from your %s.', 'pingwatcher'), $owp_link);
			echo "<br />" . __('However, if one is repeatedly failing over a long period of time, it justifies investigation and possible removal.', 'pingwatcher') . "</li>
		</ul>";

/* FURTHER INFORMATION SECTION */
		echo "<h2>" . __('Further Information', 'pingwatcher') . "</h2>";
		echo "<p>" . __('WANTED - Bug Reports', 'pingwatcher') . "<br />";
		echo __('WANTED - Translations', 'pingwatcher') . "</p><p>";
		_e('If you have any questions, problems, comments, or suggestions, please let me know.', 'pingwatcher');
		echo "</p><p>";
		printf(__('If you would like to provide a translation, please leave a comment at %s', 'pingwatcher'), '<a href=\'http://www.mittineague.com/blog/2009/04/ping-watcher-translations/\'>http://www.mittineague.com/blog/2009/04/ping-watcher-translations/</a>');
		echo "</p><p>";
		printf(__('For more information, the latest version, etc. please visit the plugin\'s page %s', 'pingwatcher'), '<a href=\'http://www.mittineague.com/dev/pw.php\'>http://www.mittineague.com/dev/pw.php</a>');
		echo "</p><p>";
		printf(__('Questions? For support, please visit the forum %s', 'pingwatcher'), '<a href=\'http://www.mittineague.com/forums/viewtopic.php?t=110\'>http://www.mittineague.com/forums/viewtopic.php?t=110</a>');
		echo " (";
		_e('registration required to post', 'pingwatcher');
		echo ")</p><p>";
		printf(__('For comments / suggestions, please visit the blog %s', 'pingwatcher'), '<a href=\'http://www.mittineague.com/blog/2009/04/ping-watcher-plugin/\'>http://www.mittineague.com/blog/2009/04/ping-watcher-plugin/</a>');
		echo "</p><h3>";
		_e('Translation Acknowledgements', 'pingwatcher');
		echo "</h3>
<!-- /* add list as translations become available ll_CC (email|URL)?=>Name (URL=>LinkText)? */ -->
		<ul>
			<li>be_BY <a href='mailto://zhr@tut.by'>Fat Cow</a>  .  .  .  .  .  <a href='http://www.fatcow.com'>Fat Cow</a></li>
			<li>de_DE Ingoal  .  .  .  .  .  <a href='http://www.ingoal.info'>Ingoal's Insight...</a></li>
			<li>es_ES Molona</li>
			<li>nl_BE <a href='mailto://fristi@gmail.com'>Kurt Van Genechten</a></li>
			<li>sv_SE Tommy Olsson  .  .  .  .  .  <a href='http://www.autisticcuckoo.net/'>The Autistic Cuckoo</a></li>
		</ul>
	</div>";
}

/* ERROR HANDLER */
function mitt_ping_errors($code, $msg, $file)
{
	if ( ($code == '2') && ( strpos($msg, 'fsockopen') !== FALSE ) && ( strpos($msg, 'connect') !== FALSE ) && ( strpos($file, 'class-IXR.php') !== FALSE ) )
	{
		$blog_date_format = get_option('date_format');
		$blog_time_format = get_option('time_format');
		$both_format = $blog_date_format . ' ' . $blog_time_format;

		$dash_info = str_replace("fsockopen() [<a href='function.fsockopen'>function.fsockopen</a>]: ", '', $msg);
		$dash_info = str_replace('(', '<br />(', $dash_info);

		$dash_info .= " ~ " . date($both_format, time()) . '<br />';

		$ping_err_arr = get_option('mitt_pw_ping_errors');

		if ( !in_array($dash_info, $ping_err_arr) )
		{
			array_unshift($ping_err_arr, $dash_info);
			$pw_err_arr_len = count($ping_err_arr);
			$pw_len_setting = get_option('mitt_pw_ping_error_length');
			if ( $pw_err_arr_len > $pw_len_setting )
			{
				$ping_err_arr = array_slice($ping_err_arr, 0, $pw_len_setting);
			}

			update_option('mitt_pw_ping_errors', $ping_err_arr);
		}
	}
	return false;
}

function mitt_pw_pe_handler()
{
	if ( ( !get_option('mitt_er_ping_error_length') ) )
	{
		set_error_handler('mitt_ping_errors', E_WARNING);
	}
}

if (function_exists('add_action'))
{
	add_action('init', 'mitt_pw_pe_handler');
	add_action('admin_init', 'mitt_pw_admin_init');
	add_action('admin_menu', 'mitt_add_pw_page');
	add_action('wp_dashboard_setup', 'add_pw_dashboard_widget');
}

if (function_exists('register_activation_hook'))
{
	register_activation_hook( __FILE__, 'mitt_pw_activate');
}

if (function_exists('register_deactivation_hook'))
{
	register_deactivation_hook( __FILE__, 'mitt_pw_deactivate');
}

if (function_exists('register_uninstall_hook'))
{
	register_uninstall_hook( __FILE__, 'mitt_pw_uninstall');
}
?>